import os
import pylab
import dadi
import re
from dadi import Numerics, PhiManip, Integration
from dadi.Spectrum_mod import Spectrum

#load data
#os.chdir("/Users/cj/Dropbox/Passerina_ciris/dadi/")
os.chdir("/media/burke/bigMac/Dropbox/Passerina_ciris/dadi/")
dd=dadi.Misc.make_data_dict('data/pabu30s_k3_c20.dadi')
data=dadi.Spectrum.from_data_dict(dd,['pop1','pop2','pop3'],[10,10,10],
                                polarized=False)

#define demographic model
def IM_k3(params, ns, pts):
    N1,N2,N3,T2,T1,m13,m31,m23,m32 = params
    xx = dadi.Numerics.default_grid(pts)
    phi = dadi.PhiManip.phi_1D(xx)
    phi = dadi.PhiManip.phi_1D_to_2D(xx,phi)
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1=N1, nu2=N2, m12=0., m21=0.)
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    phi = dadi.Integration.three_pops(phi, xx, T1,
                                       nu1=N1, nu2=N2, nu3=N3,
                                       m12=0, m13=m13,
                                       m21=0, m23=m23,
                                       m31=m31, m32=m32)
    fs = dadi.Spectrum.from_phi(phi,ns,(xx,xx,xx),
                                pop_ids=['pop1', 'pop2', 'pop3'])
    return fs

#set likelihood search parameters
ns=data.sample_sizes
func=IM_k3
func_ex=dadi.Numerics.make_extrap_log_func(func)
upper_bound=[50,10,10,10,10,20,20,20,20]
lower_bound=[1e-5,1e-5,1e-5,1e-5,1e-5,0,0,0,0,]
pts=[40,60,80]

#output file formatting
# k3out=open('k3popt_split23.txt','a')
# k3out.write("N1"+"\t"+
#             "N2"+"\t"+
#             "N3"+"\t"+
#             "T2"+"\t"+
#             "T1"+"\t"+
#             "m13"+"\t"+
#             "m31"+"\t"+
#             "m23"+"\t"+
#             "m32"+"\t"+
#             "Theta"+"\t"+
#             "ll_model"+"\t"+
#             "L"+"\n")
# k3out.close()

#run 10 optimizations w randomized starting positions
for i in range(20):
    p0=[5.41937034,0.45898727,3.53451369,1.78001861,0.349417,4.54681965,0.50083448,4.1625115,1.39251801]
    #p0=[30.2766,.2370,9.391,.406,0.453,5.346,0.678,4.626,2.054]
    #p0=[5,.3,1.3,.1,.2,2.7,2.2,6.3,4.7]
    #p0=[4.5,.33,3.7,1.9,.3,4.6,.5,4.0,1.4]
    p0=dadi.Misc.perturb_params(p0,2,upper_bound=upper_bound,lower_bound=lower_bound)
    popt=dadi.Inference.optimize_log(p0,data,func_ex,pts,
                                lower_bound=lower_bound,
                                upper_bound=upper_bound,
                                verbose=1,
                                maxiter=10)
    model = func_ex(popt, ns, pts)
    # Likelihood of the data given the model AFS.
    ll_model = dadi.Inference.ll_multinom(model, data)
    print('Maximum log composite likelihood: {0}'.format(ll_model))
    # The optimal value of theta given the model.
    theta = dadi.Inference.optimal_sfs_scaling(model, data)
    print('Optimal value of theta: {0}'.format(theta))

    #converting parameter estimates to real values
    L=(data.S()/14603)*4128*92 #scale total seq length of 5671*92bp (for c48) by fraction of all SNP's in actual analysis
    Nref = theta/(4*3.44e-9*L) #theta=4nu, with u=2.19e-9sub/site/yr & gen time = 2 yrs
    N1=popt[0]*Nref
    N2=popt[1]*Nref
    N3=popt[2]*Nref
    T1=popt[4]*2*Nref #C/W split
    T2=popt[3]*2*Nref+T1 #E/W split
    m13=(popt[5]/(2*Nref))
    m31=(popt[6]/(2*Nref))
    m23=(popt[7]/(2*Nref))
    m32=(popt[8]/(2*Nref))
    realparams=[Nref,N1,N2,N3,T1,T2,m13,m31,m23,m32,data.Fst()]
    roundedparams=['%.3f' % e for e in realparams]
    print(['Nref','N1','N2','N3','T1','T2','m13','m31','m23','m32','Fst'])
    print(roundedparams)
    k3out=open('k3popt_split23.txt','a')
    k3out.write(str(popt[0])+'\t'+
                str(popt[1])+'\t'+
                str(popt[2])+'\t'+
                str(popt[3])+'\t'+
                str(popt[4])+'\t'+
                str(popt[5])+'\t'+
                str(popt[6])+'\t'+
                str(popt[7])+'\t'+
                str(popt[8])+'\t'+
                str(theta)+'\t'+
                str(ll_model)+'\t'+
                str(L)+'\n')
    k3out.close()
    # pylab.figure(1)
    # dadi.Plotting.plot_3d_comp_multinom(model, data, vmin=1, resid_range=3,pop_ids =('W','E','C'))
