import os
import pylab
import dadi
import re
import random
from dadi import Numerics, PhiManip, Integration
from dadi.Spectrum_mod import Spectrum


#define demographic model
def IM_k3(params, ns, pts):
    N1,N2,N3,T2,T1,m13,m31,m23,m32 = params
    xx = dadi.Numerics.default_grid(pts)
    phi = dadi.PhiManip.phi_1D(xx)
    phi = dadi.PhiManip.phi_1D_to_2D(xx,phi)
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1=N1, nu2=N2, m12=0., m21=0.)
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    phi = dadi.Integration.three_pops(phi, xx, T1,
                                       nu1=N1, nu2=N2, nu3=N3,
                                       m12=0, m13=m13,
                                       m21=0, m23=m23,
                                       m31=m31, m32=m32)
    fs = dadi.Spectrum.from_phi(phi,ns,(xx,xx,xx),
                                pop_ids=['pop1', 'pop2', 'pop3'])
    return fs


#os.chdir("/Users/cj/Dropbox/Passerina_ciris/dadi/bootstrap_30s/")
os.chdir("/media/burke/bigMac/Dropbox/Passerina_ciris/dadi/bootstrap_30s")
files=os.listdir("./")
random.shuffle(files)

k3out=open('../k3popt_split23_BS.txt','a')
k3out.write('N1'+'\t'+
            'N2'+'\t'+
            'N3'+'\t'+
            'T2'+'\t'+
            'T1'+'\t'+
            'm13'+'\t'+
            'm31'+'\t'+
            'm23'+'\t'+
            'm32'+'\t'+
            'theta'+'\t'+
            'll_model'+'\t'+
            'L'+'\n')
k3out.close()

for i in files:
    print(i)
    dd=dadi.Misc.make_data_dict(i)
    data=dadi.Spectrum.from_data_dict(dd,['pop1','pop2','pop3'],[10,10,10],
                                polarized=False)
    ns=data.sample_sizes
    func=IM_k3
    func_ex=dadi.Numerics.make_extrap_log_func(func)
    upper_bound=[50,10,10,10,10,20,20,20,20]
    lower_bound=[1e-5,1e-5,1e-5,1e-5,1e-5,0,0,0,0,]
    pts=[20,40,60]
    #p0=[6.38281066568,0.470896615036,2.76072935516,0.432234471826,3.05193123493,2.23200648157,2.7167528944,3.96477555433,1.31818622075] #split 1 optimized params
    p0=[4.92950525578,0.358371414429,2.62484335557,0.942038425618,1.98631409385,2.8195082371,2.71679612246,5.45140503512,1.52433229427] #split 2 optimized params
    popt=dadi.Inference.optimize_log(p0,data,func_ex,pts,
                                lower_bound=lower_bound,
                                upper_bound=upper_bound,
                                verbose=1,
                                maxiter=5)
    model = func_ex(popt, ns, pts)
    # Likelihood of the data given the model AFS.
    ll_model = dadi.Inference.ll_multinom(model, data)
    print('Maximum log composite likelihood: {0}'.format(ll_model))
    # The optimal value of theta given the model.
    theta = dadi.Inference.optimal_sfs_scaling(model, data)
    print('Optimal value of theta: {0}'.format(theta))

    #converting parameter estimates to real values
    L=(data.S()/14603)*4128*92
    Nref = theta/(4*3.44e-9*L)
    N1=popt[0]*Nref
    N2=popt[1]*Nref
    N3=popt[2]*Nref
    T1=popt[4]*2*Nref
    T2=popt[3]*2*Nref+T1
    m13=(popt[5]/(2*Nref))
    m31=(popt[6]/(2*Nref))
    m23=(popt[7]/(2*Nref))
    m32=(popt[8]/(2*Nref))
    realparams=[Nref,N1,N2,N3,T1,T2,m13,m31,m23,m32,data.Fst()]
    roundedparams=['%.3f' % e for e in realparams]
    print(['Nref','N1','N2','N3','T1','T2','m13','m31','m23','m32','Fst'])
    print(roundedparams)
    k3out=open('../k3popt_split23_BS.txt','a')
    k3out.write(str(popt[0])+'\t'+
                str(popt[1])+'\t'+
                str(popt[2])+'\t'+
                str(popt[3])+'\t'+
                str(popt[4])+'\t'+
                str(popt[5])+'\t'+
                str(popt[6])+'\t'+
                str(popt[7])+'\t'+
                str(popt[8])+'\t'+
                str(theta)+'\t'+
                str(ll_model)+'\t'+
                str(L)+'\n')
    k3out.close()
